Chart.defaults.global.defaultFontSize = 18;
Chart.defaults.global.defaultFontStyle = "bold";

var userChartConfig = {
    name: "UserChart",
    // The type of chart we want to create
    type: 'line',

    // The data for our dataset
    data: {
        labels: [],
        datasets: [{
            label: 'Benutzerzahlen',
            lineTension: 0.3,
            backgroundColor: "rgba(78, 115, 223, 0.05)",
            borderColor: "rgba(78, 115, 223, 1)",
            pointRadius: 3,
            pointBackgroundColor: "rgba(78, 115, 223, 1)",
            pointBorderColor: "rgba(78, 115, 223, 1)",
            pointHoverRadius: 3,
            pointHoverBackgroundColor: "rgba(78, 115, 223, 1)",
            pointHoverBorderColor: "rgba(78, 115, 223, 1)",
            pointHitRadius: 10,
            pointBorderWidth: 2,
            data: []

        }]
    },

    // Configuration options go here
    options: {
        maintainAspectRatio: false,
        layout: {
            padding: {
                left: 10,
                right: 25,
                top: 25,
                bottom: 0
            }
        },
        scales: {
            xAxes: [{
                time: {
                    unit: 'date'
                },
                gridLines: {
                    display: false,
                    drawBorder: false
                },
                ticks: {
                    maxTicksLimit: 7
                }
            }],
            yAxes: [{
                ticks: {
                    suggestedMin: 0,
                    maxTicksLimit: 5,
                    padding: 10
                    // Include a dollar sign in the ticks
                    //   callback: function(value, index, values) {
                    //     return '$' + number_format(value);
                    //   }
                },
                gridLines: {
                    color: "rgb(234, 236, 244)",
                    zeroLineColor: "rgb(234, 236, 244)",
                    drawBorder: false,
                    borderDash: [2],
                    zeroLineBorderDash: [2]
                }
            }],
        },
        legend: {
            display: false
        },
        tooltips: {
            backgroundColor: "rgb(255,255,255)",
            bodyFontColor: "#858796",
            titleMarginBottom: 10,
            titleFontColor: '#6e707e',
            titleFontSize: 14,
            borderColor: '#dddfeb',
            borderWidth: 1,
            xPadding: 15,
            yPadding: 15,
            displayColors: false,
            intersect: false,
            mode: 'index',
            caretPadding: 10,
            callbacks: {
                label: function (tooltipItem, chart) {
                    if (tooltipItem.index == 0) {
                        return tooltipItem.value + ` (+${tooltipItem.value})`;
                    } else {
                        let previousVal = chart.datasets[tooltipItem.datasetIndex].data[tooltipItem.index - 1]
                        let currentVal = parseInt(tooltipItem.value, 10);
                        let difference = currentVal - previousVal;
                        // LogInUnity(`${previousVal} - ${currentVal} = ${difference}`);
                        return tooltipItem.value + ` (${difference >= 0 ? "+" : ""}${difference})`;
                    }
                }
            }
        }
    }
};

var serverLoadChartConfig = {
    name: "ServerLoadChart",
    // The type of chart we want to create
    type: 'line',

    // The data for our dataset
    data: {
        labels: [],
        datasets: [{
            label: 'Serverauslastung',
            pointRadius: 0,
            backgroundColor: 'rgba(149, 40, 180, 0.05)',
            borderColor: 'rgb(149, 40, 180)',
            showLine: true,
            data: []
        }]
    },

    // Configuration options go here
    options: {
        maintainAspectRatio: false,
        layout: {
            padding: {
                left: 10,
                right: 25,
                top: 25,
                bottom: 0
            }
        },
        scales: {
            xAxes: [{
                time: {
                    unit: 'date'
                },
                gridLines: {
                    color: "rgb(234, 236, 244)",
                    zeroLineColor: "rgb(234, 236, 244)",
                    drawBorder: false
                },
                ticks: {
                    maxTicksLimit: 7,
                    color: "rgb(234, 0, 0)"
                }
            }],
            yAxes: [{
                ticks: {
                    suggestedMin: 0,
                    suggestedMax: 100,
                    maxTicksLimit: 5,
                    padding: 10
                    //   // Include a dollar sign in the ticks
                    //   callback: function(value, index, values) {
                    //     return '$' + number_format(value);
                    //   }
                },
                gridLines: {
                    color: "rgb(234, 236, 244)",
                    zeroLineColor: "rgb(234, 236, 244)",
                    drawBorder: false,
                    borderDash: [2],
                    zeroLineBorderDash: [2]
                }
            }],
        },
        legend: {
            display: false
        },
        tooltips: {
            enabled: false
        }
    }
};
var serverStorageChartConfig = {
    name: "ServerStorageChart",
    // The type of chart we want to create
    type: 'doughnut',
    // The data for our dataset
    data: {
        labels: ['Belegt', 'Frei'],
        datasets: [{
            backgroundColor: ['rgb(0, 114, 206)', 'rgb(170, 170, 170)'],
            data: [80, 20]
        }]
    },

    // Configuration options go here
    options: {
        maintainAspectRatio: false,
        tooltips: {
          backgroundColor: "rgb(255,255,255)",
          bodyFontColor: "#858796",
          borderColor: '#dddfeb',
          borderWidth: 1,
          xPadding: 15,
          yPadding: 15,
          displayColors: false,
          caretPadding: 10,
        },
        legend: {
          display: false
        },
        cutoutPercentage: 63,
      },
};

function UpdateDiagrams(serverStatus) {
    let printDate = "";
    // Server Load 

    // if the date changed
    if (serverStatus.Date != currentDate) {
        printDate = serverStatus.Date;
        currentDate = serverStatus.Date;

        // Users should only get updated once per day
        addData(UserChart, printDate, serverStatus.Users);

    }
    addData(ServerLoadChart, printDate, serverStatus.CurrentServerLoad);
    if (ServerLoadChart.data.datasets[0].data.length > 50) {
        removeFirstData(ServerLoadChart);
    }
    ServerLoadChart.update();
    // Storage
    if (ServerStorageChart.data.datasets[0].data[0] != serverStatus.StorageUsed) {
        ServerStorageChart.data.datasets[0].data[0] = serverStatus.StorageUsed;
        ServerStorageChart.data.datasets[0].data[1] = serverStatus.StorageCapacity - serverStatus.StorageUsed;
        ServerStorageChart.update();
        document.getElementById("usedStorage").innerText = serverStatus.StorageUsed;
        document.getElementById("freeStorage").innerText = serverStatus.StorageCapacity - serverStatus.StorageUsed;
    }

    // Top row cards
    document.getElementById('usersNumber').innerText = serverStatus.Users;
    document.getElementById('dailyIncome').innerText = serverStatus.dailyIncome;



    // Upgrade Buttons
    document.getElementById('StorageUpgrAmnt1').innerText = serverStatus.upgradeOptions.storage1Amount;
    document.getElementById('StorageUpgrCost1').innerText = serverStatus.upgradeOptions.storage1Cost;
    if (serverStatus.upgradeOptions.storage1Cost > serverStatus.availableMoney) { 
        document.getElementById('StorageUpgrCost1').parentElement.parentElement.classList.add('disabled');
    } else {
        document.getElementById('StorageUpgrCost1').parentElement.parentElement.classList.remove('disabled');
    }

    document.getElementById('StorageUpgrAmnt2').innerText = serverStatus.upgradeOptions.storage2Amount;
    document.getElementById('StorageUpgrCost2').innerText = serverStatus.upgradeOptions.storage2Cost;
    if (serverStatus.upgradeOptions.storage2Cost > serverStatus.availableMoney) { 
        document.getElementById('StorageUpgrCost2').parentElement.parentElement.classList.add('disabled');
    } else {
        document.getElementById('StorageUpgrCost2').parentElement.parentElement.classList.remove('disabled');
    }

    document.getElementById('PerfUpgrCost1').innerText = serverStatus.upgradeOptions.performance1Cost;
    if (serverStatus.upgradeOptions.performance1Cost > serverStatus.availableMoney) { 
        document.getElementById('PerfUpgrCost1').parentElement.parentElement.classList.add('disabled');
    } else {
        document.getElementById('PerfUpgrCost1').parentElement.parentElement.classList.remove('disabled');
    }

    document.getElementById('PerfUpgrCost2').innerText = serverStatus.upgradeOptions.performance2Cost;
    if (serverStatus.upgradeOptions.performance2Cost > serverStatus.availableMoney) { 
        document.getElementById('PerfUpgrCost2').parentElement.parentElement.classList.add('disabled');
    } else {
        document.getElementById('PerfUpgrCost2').parentElement.parentElement.classList.remove('disabled');
    }    


    // Logging
    let formatted = [];
    // printError("From Game: " + serverStatus.logs + ";;;");
    serverStatus.logs.forEach(x => {
        formatted.push(x.split('|'));
    });

    logtable.rows().remove();
    logtable.rows.add(formatted);
    logtable.draw();


    // Overview
    document.getElementById('providerName').innerText = serverStatus.hoster;
    document.getElementById('serverType').innerText = serverStatus.serverType;
    
    document.getElementById('serverLoadPercent').innerText = serverStatus.CurrentServerLoad;
    let storagePercentage =  Math.floor((serverStatus.StorageUsed / serverStatus.StorageCapacity)*100);
    document.getElementById('storageLoadPercent').innerText = storagePercentage;
    document.getElementById('storageLoadProgressBar').style.width = `${storagePercentage}%`;
    
    document.getElementById('usersNumberStartpage').innerText = serverStatus.Users;
    document.getElementById('dailyIncomeStartpage').innerText = serverStatus.dailyIncome;




}

function displayContent(contentClass) {
    let pages = document.getElementsByClassName('monitoringpage');
    for (let i = 0; i < pages.length; i++) {
        let page = pages[i];
        if (page.classList.contains(contentClass) || contentClass == "all") {
            page.style.display = "";
        } else {
            page.style.display = "none";
        }
    }
}

function UpgradeServer(system, option) {
    var request;
    if (system == "storage" && option == 1) {
        request = '{"queryTypeString":"MONITORING_START_UPGRADE", "queryParam":"storage1"}';
    } else if (system == "storage" && option == 2) {
        request = '{"queryTypeString":"MONITORING_START_UPGRADE", "queryParam":"storage2"}';
    }

    if (system == "performance" && option == 1) {
        request = '{"queryTypeString":"MONITORING_START_UPGRADE", "queryParam":"performance1"}';
    } else if (system == "performance" && option == 2) {
        request = '{"queryTypeString":"MONITORING_START_UPGRADE", "queryParam":"performance2"}';
    }

    makeCefQuery(request, () => { }, (error_code, error_message) => { printError(error_code + error_message) });
}



var UserChart = new Chart(document.getElementById("UserChart").getContext('2d'), userChartConfig);
var ServerLoadChart = new Chart(document.getElementById("ServerLoadChart").getContext('2d'), serverLoadChartConfig);
ServerLoadChart.options.animation = false;
var ServerStorageChart = new Chart(document.getElementById("ServerStorageChart").getContext('2d'), serverStorageChartConfig);

var logtable = $('#logtable').DataTable({
    "searching": false,
    "paging": false,
    "bInfo": false,
    columns: [
        { title: "Severity" , "width": "10%" },
        { title: "Date"     , "width": "20%" },
        { title: "Message"  , "width": "70%" },
    ]
});

var currentDate;
var serverStatus;
var intervalID = window.setInterval(function () {
    GetServerStatus((serverStatus) => {
        // printError(JSON.stringify(serverStatus));
        displayContent(serverStatus.displayPage)
        if (! serverStatus.paused) {
            UpdateDiagrams(serverStatus);
        }
    });
}, 250);
